class Ability
{
    maxCharges = 1;
    charges = 1;
    player = null;
    useTime = 0;
    cooldown = 5;
    shareWithPatient = false;

    function DoAction(player) {}
    function GetDisplayName()
    {
        local result;
        if (maxCharges > 1)
            result = GetNameInner() + " (" + charges + ")";
        else
            result = GetNameInner();
        if (shareWithPatient && player.GetPlayerClass() == TF_CLASS.MEDIC)
            result += " (Sharable)"
        return result;
    }
    function GetNameInner() {}

    function constructor(player)
    {
        this.player = player;
        charges = maxCharges;
    }

    function TryDoAction()
    {
        if (Time() - useTime < 0)
            return;
        useTime = Time() + cooldown;
        EmitSoundOn("MVM.PlayerUsedPowerup", player);
        DoAction(player);
        if (shareWithPatient && player.GetPlayerClass() == TF_CLASS.MEDIC)
        {
            local medigun = player.ReturnWeapon(TF_WEAPONSLOTS.SECONDARY);
            if (medigun != null)
            {
                local patient = NetProps.GetPropEntity(medigun, "m_hHealingTarget")
                if (patient != null)
                {
                    EmitSoundOn("MVM.PlayerUsedPowerup", patient);
                    DoAction(patient);
                }
            }
        }

        if (--charges <= 0)
            playerAbilities[player] <- null
    }
}

class OpeningAbility extends Ability
{
    cooldown = 1337;

    function constructor(player)
    {
        base.constructor(player);
        EmitSoundOn("ui.crate_open_short", player);
        EmitSoundOn("ui.crate_open_short", player);
        RunWithDelay("GrantRandomAbilityToPlayer(activator)", player, 3);
    }

    function DoAction(player)
    {
    }

    function GetDisplayName()
    {
        return "Opening...";
    }
}

class OverhealAbility extends Ability
{
    shareWithPatient = true;

    function DoAction(player)
    {
        player.AddCondEx(Constants.ETFCond.TF_COND_INVULNERABLE_CARD_EFFECT, 1, player);
        RunWithDelay("activator.AddCondEx(Constants.ETFCond.TF_COND_HALLOWEEN_QUICK_HEAL, 3, activator)", player, 1)
    }

    function GetNameInner()
    {
        return "Overheal";
    }
}

class KritzAbility extends Ability
{
    shareWithPatient = true;

    function DoAction(player)
    {
        player.AddCondEx(Constants.ETFCond.TF_COND_CRITBOOSTED_CARD_EFFECT, 5, player);
    }

    function GetNameInner()
    {
        return "Kritz";
    }
}

class EscapeAbility extends Ability
{
    tpTargets = [];

    function constructor(player)
    {
        local tpTarget = null;
        while (tpTarget = Entities.FindByClassname(tpTarget, "env_entity_maker"))
            tpTargets.push(tpTarget);
        if (tpTargets.len() == 0)
            GrantRandomAbilityToPlayer(player);
        else
            base.constructor(player);
    }

    function DoAction(player)
    {
        local index = RandomInt(0, tpTargets.len()-1);
        DispatchParticleEffect("teleported_red", player.GetOrigin(), Vector(0,90,0));
        DispatchParticleEffect("wrenchmotron_teleport_beam", player.GetOrigin(), Vector(0,90,0));
        EmitSoundOn("Building_Teleporter.Send",player);
        player.Teleport(true, tpTargets[index].GetOrigin(), true, QAngle(0,0,0), true, Vector(0,0,0))
        DispatchParticleEffect("teleportedin_red", player.GetOrigin(), Vector(0,90,0));
    }

    function GetNameInner()
    {
        return "Escape";
    }
}

class SpeedAbility extends Ability
{
    maxCharges = 2;
    cooldown = 4.8;
    shareWithPatient = true;

    function DoAction(player)
    {
        player.AddCustomAttribute("move speed bonus", 1.7, 5);
        player.AddCondEx(Constants.ETFCond.TF_COND_SPEED_BOOST, 5, player);
    }

    function GetNameInner()
    {
        return "Speed";
    }
}

class SuckerJumpAbility extends Ability
{
    maxCharges = 2;
    cooldown = 1;

    function DoAction(player)
    {
        vsh_vscript.haleBraveJumpAbility.Perform(player);
        player.AddCustomAttribute("cancel falling damage", 1, 5);
        EmitSoundOn("TFPlayer.AirBlastImpact", player);
        EmitSoundOn("Taunt.Taunt.SpringRiderSqueak2", player);
    }

    function GetNameInner()
    {
        return "Sucker Jump";
    }
}

class InvisAbility extends Ability
{
    function DoAction(player)
    {
        player.AddCondEx(Constants.ETFCond.TF_COND_STEALTHED_USER_BUFF, 15, player);
    }

    function GetNameInner()
    {
        return "Invisibility";
    }
}

/*class BlastResistanceAbility extends Ability
{
    charges = 1;

    function DoAction()
    {
        player.AddCondEx(Constants.ETFCond.TF_COND_SPEED_BOOST, 5, player);
        //player.AddCustomAttribute("damage bonus", 0.2, 5);
    }

    function GetNameInner()
    {
        return "Blast Resistance";
    }
}*/